import base64
import os
import subprocess

from charmhelpers import fetch
from charmhelpers.core import (
    hookenv,
    host,
)
from charmhelpers.core.services import helpers
from charmhelpers.payload import (archive, execd)


REQUIRED_PACKAGES = [
    'python-virtualenv', 'python3-dev',
]

WSGI_USER = 'www-data'
WSGI_GROUP = 'www-data'

config = hookenv.config()

SERVICE_DIR = '/srv/{}/adt-cloud-service'.format(config['environment'])
LOG_DIR = os.path.join(SERVICE_DIR, 'logs')


def log_start(service_name):
    hookenv.log('adt-cloud-service starting')


def basenode(service_name):
    hookenv.log("Executing basenode")
    execd.execd_preinstall()


def install_packages(service_name):
    hookenv.log('Installing dependencies...')
    fetch.add_source('ppa:canonical-ci-engineering/ci-airline-phase-0')
    fetch.configure_sources(update=True)
    fetch.apt_install(REQUIRED_PACKAGES, fatal=True)


def get_cloud_service_from_tarball(service_name):
    files_dir = os.path.join(hookenv.charm_dir(), 'files')
    tarball = os.path.join(files_dir, 'adt-cloud-service.tgz')
    if not os.path.exists(SERVICE_DIR):
        hookenv.log('Installing the code for the first time from tarball')
        archive.extract_tarfile(tarball, os.path.dirname(SERVICE_DIR))


def get_config_file(service_name):
    config_file = config.get('config-file')
    if config_file is not None:
        with open(os.path.join(SERVICE_DIR, 'adt-service.conf'), 'w') as f:
            f.write(base64.b64decode(config_file))


def install_python_packages(service_name):
    env_dir = os.path.join(SERVICE_DIR, 've')
    if os.path.exists(env_dir):
        hookenv.log('Service venv already exists, nothing to do ...')
        return

    hookenv.log('Installing python packages into the venv ...')
    subprocess.check_call(['virtualenv', '-p', 'python3', env_dir])

    pip_cache = os.path.join(hookenv.charm_dir(), 'files', 'pip-cache')
    requirements = os.path.join(SERVICE_DIR, 'requirements.txt')

    hookenv.log('Installing from download cache.')
    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install',
                           '--no-index',
                           '--find-links={}'.format(pip_cache),
                           '-r', requirements])

    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install', '--no-deps', '-e', SERVICE_DIR])


def prepare_for_wsgi(service_name):
    host.adduser(WSGI_USER)
    host.add_group(WSGI_GROUP)
    host.add_user_to_group(WSGI_USER, WSGI_GROUP)
    if not os.path.exists(LOG_DIR):
        host.mkdir(LOG_DIR, WSGI_USER, WSGI_GROUP, perms=0755)


def publish_website_relation_data(service_name):
    # Due to Bug #1409763, this functionality is as action rather than the
    # provided_data below.
    for relid in hookenv.relation_ids('website'):
        hookenv.log(30 * '*')
        hookenv.log('Hack for #1409763: {}'.format(relid))
        hookenv.log(30 * '*')
        hookenv.relation_set(
            relid, host=hookenv.unit_get('private-address'), port=8000)


class WebsiteRelation(helpers.HttpRelation):

    def provide_data(self):
        hookenv.log('Providing Website relation data.')
        data = super(WebsiteRelation, self).provide_data()
        data['port'] = 8000
        return data
